#ifndef __SLIDEFACE_H__
#define __SLIDEFACE_H__

#include "slideprimitive.h"

class CSLIDEWorld;
class CSLIDEPoint;
class CSLIDESurface;

//////////////////////////////////////////////////////////////////////
// CSLIDEFace Class
//

class CSLIDEFaceIter;

class CSLIDEFace : public CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Friends
//
public:
  friend class CSLIDEFaceIter;

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEFace();
  virtual ~CSLIDEFace();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcID, 
		    CListInt *plistPointIDs);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEPrimitiveIter *AllocateIter();
  virtual VOID FreeIter(CSLIDEPrimitiveIter *pCSLIDEFaceIter);

//////////////////////////////////////////////////
// Geometric Computations
//
public:
////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  VOID InsertIntoBound(CBound *pCBound);
  virtual VOID Preprocess();

protected:
  VOID CheckDynamic();
  VOID UpdateState();

//////////////////////////////////////////////////
// Rendering
//
public:
  virtual VOID Render(const Matrix &mGvp_proj, 
		      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
		      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
		      CBound *pCBoundClip, CListInt *plistLights,
		      CSLIDESurface *pCSLIDESurface,
		      SLIDEShadingType eShading, SLIDELODType eLOD, 
		      SLIDESolidType eSolid);

protected:
  virtual VOID ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface);
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline VOID SetSurfaceID(CHAR *pcSurfaceID);

  virtual UINT_32 GetPointCount();
  virtual inline BOOL GetBounded();
  virtual inline VOID SetBounded(BOOL bBounded);
  virtual inline BOOL GetBoundDynamic();
  virtual inline VOID SetBoundDynamic(BOOL bBoundDynamic);

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static CListInt *m_plistVertices;
  static UINT_32 m_uiCount;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcSurfaceID;
  CSLIDESurface *m_pCSLIDESurface; // The surface which gives color
  CListInt *m_plistPointIDs;
  CListInt m_listCSLIDEPoint;      // List of CSLIDEPoints

  BOOL m_bBounded;  // Does this node actually have geometry in it (not a camera)
  BOOL m_bBoundDynamic; // Can the geometry change dynamically?
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDEFace::SetSurfaceID(CHAR *pcSurfaceID)
{
  m_pcSurfaceID = pcSurfaceID;
}

inline BOOL CSLIDEFace::GetBounded()
{
  return m_bBounded;
}

inline VOID CSLIDEFace::SetBounded(BOOL bBounded)
{
  m_bBounded = bBounded;
}

inline BOOL CSLIDEFace::GetBoundDynamic()
{
  return m_bBoundDynamic;
}

inline VOID CSLIDEFace::SetBoundDynamic(BOOL bBoundDynamic)
{
  m_bBoundDynamic = bBoundDynamic;
}

//////////////////////////////////////////////////////////////////////
// CSLIDEFaceIter Class
//

class CSLIDEFaceIter : public CSLIDEPrimitiveIter
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEFaceIter();
  virtual ~CSLIDEFaceIter();
  virtual VOID Init(CSLIDEFace *pCSLIDEFace);
  virtual VOID Uninit();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEPoint *PeekFirstCSLIDEPoint();
  virtual CSLIDEPoint *PeekNextCSLIDEPoint();
  virtual CSLIDEPoint *PeekLastCSLIDEPoint();
  virtual CSLIDEPoint *PeekPrevCSLIDEPoint();

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CListIterInt m_iter;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//


#endif // __SLIDEFACE_H__
