#ifndef __SLIDENODE_H__
#define __SLIDENODE_H__

#include "slideprimitive.h"

class CSLIDEWorld;
class CSLIDECamera;
class CSLIDELight;
class CSLIDEFace;
class CSLIDESurface;

//////////////////////////////////////////////////////////////////////
// CSLIDENode Class
//

class CSLIDENode : public CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Friends
//
public:
  friend class CSLIDENodeIter;

//////////////////////////////////////////////////
// Enumerated Types
//
public:
enum NodeType {ntObject, ntGroup, ntInstance, ntCamera, ntLight, ntSize};

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDENode();
  virtual ~CSLIDENode();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Iteration Functions
//
public:

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual CSLIDECamera *FindCamera(CListIterInt *piterIDs, 
				   CListInt *plistCSLIDEInstance) = 0;
  virtual CSLIDELight *FindLight(CListIterInt *piterIDs, 
				 CListInt *plistCSLIDEInstance) = 0;

////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  //virtual VOID Preprocess(); // Update all objects once, this is the only
			     // time that static matrices are processed
  virtual VOID PreprocessGraph(); // Runs through the Scene Graph once to
				  // establish which subtrees are dynamic
  //virtual VOID Update();
  virtual VOID UpdateGraph(); // Runs through Scene Graph
                              // and recalculates any changed bounds
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////
  virtual VOID InsertIntoBound(const Matrix &m, CBound &bound);

protected:
  virtual VOID CheckDynamic() = 0;
  virtual VOID UpdateState() = 0;
  virtual VOID CalculateBound();
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Rendering
//
public:
////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  virtual VOID Render(const Matrix &mGvp_proj, 
		      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
		      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
		      CBound *pCBoundClip, 
		      CListInt *plistLights,
		      CSLIDESurface *pCSLIDESurface,
		      SLIDEShadingType eShading, 
		      SLIDELODType eLOD);
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

protected:
  virtual VOID ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface);
  virtual VOID ModifyShadingFlag(SLIDEShadingType &eShading);
  virtual VOID ModifyLODFlag(SLIDELODType &eLOD);

  virtual VOID RenderGeometry(const Matrix &mGvp_proj, 
			      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
			      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
			      CBound *pCBoundClip, 
			      CListInt *plistLights,
			      CSLIDESurface *pCSLIDESurface,
			      SLIDEShadingType eShading, 
			      SLIDELODType eLOD);

public:
  virtual VOID RenderBound(const Matrix &mGvp_proj, const Matrix &mGproj_vrc, 
			   const Matrix &mGvrc_obj);

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile) = 0;

//////////////////////////////////////////////////
// Class Accessor Functions
//
public:
  static inline BOOL GetRenderBound();
  static inline VOID SetRenderBound(BOOL bRenderBound);
  static inline BOOL GetColorDynamicBounds();
  static inline VOID SetColorDynamicBounds(BOOL bColorDynamicBounds);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  virtual inline VOID SetSurfaceID(CHAR *pcSurfaceID);
  virtual inline VOID SetShading(DynInt *pdyniShading);
  virtual inline VOID SetLOD(DynInt *pdyniLOD);

  virtual inline NodeType GetNodeType() = 0;

  virtual inline const CBound *GetCBound();
  virtual inline BOOL GetBounded();
  virtual inline VOID SetBounded(BOOL bBounded);
  virtual inline BOOL GetBoundDynamic();
  virtual inline VOID SetBoundDynamic(BOOL bBoundDynamic);
  virtual inline BOOL GetBoundValid();
  virtual inline VOID SetBoundValid(BOOL bBoundValid);
  virtual inline BOOL GetBoundChanged();
  virtual inline VOID SetBoundChanged(BOOL bBoundChanged);

  virtual inline BOOL GetBoundEmpty();

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static BOOL m_bRenderBound;  // Whether to render the bounding boxes
  static BOOL m_bColorDynamicBounds;  // Whether to color dynamic bounding boxes red

//////////////////////////////////////////////////
// Member Variables
//
protected:
  BOOL m_bBounded;  // Does this node actually have geometry in it (not a camera)
  BOOL m_bBoundDynamic; // Can the geometry change dynamically?      
  BOOL m_bBoundValid;   // Has the bound been computed on this update?
  BOOL m_bBoundChanged; // Did the bound change when recomputing it this update?
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//
inline BOOL CSLIDENode::GetRenderBound()
{
  return m_bRenderBound;
}

inline VOID CSLIDENode::SetRenderBound(BOOL bRenderBound)
{
  m_bRenderBound = bRenderBound;
}

inline BOOL CSLIDENode::GetColorDynamicBounds()
{
  return m_bColorDynamicBounds;
}

inline VOID CSLIDENode::SetColorDynamicBounds(BOOL bColorDynamicBounds)
{
  m_bColorDynamicBounds = bColorDynamicBounds;
}

inline VOID CSLIDENode::SetSurfaceID(CHAR * /*pcSurfaceID*/)
{
}

inline VOID CSLIDENode::SetShading(DynInt * /*pdyniShading*/)
{
}

inline VOID CSLIDENode::SetLOD(DynInt * /*pdyniLOD*/)
{
}

inline const CBound *CSLIDENode::GetCBound()
{
  return NULL;
}

inline BOOL CSLIDENode::GetBounded()
{
  return TRUE;
}

inline VOID CSLIDENode::SetBounded(BOOL bBounded)
{
  m_bBounded = bBounded;
}

inline BOOL CSLIDENode::GetBoundDynamic()
{
  return m_bBoundDynamic;
}

inline VOID CSLIDENode::SetBoundDynamic(BOOL bBoundDynamic)
{
  m_bBoundDynamic = bBoundDynamic;
}

inline BOOL CSLIDENode::GetBoundValid()
{
  return m_bBoundValid;
}

inline VOID CSLIDENode::SetBoundValid(BOOL bBoundValid)
{
  m_bBoundValid = bBoundValid;
}

inline BOOL CSLIDENode::GetBoundChanged()
{
  return m_bBoundChanged;
}

inline VOID CSLIDENode::SetBoundChanged(BOOL bBoundChanged)
{
  m_bBoundChanged = bBoundChanged;
}

inline BOOL CSLIDENode::GetBoundEmpty()
{
  // This is right for cameras and lights
  return TRUE;
}

#endif // __SLIDENODE_H__
