#ifndef __SLIDEPRIMITIVE_H__
#define __SLIDEPRIMITIVE_H__

#include "genlib.h"
#include "datalib.h"
#include "mathlib.h"
#include "bound.h"

#include "slide.h"
#include "gillib.h"

#include "dynamics.h"

//////////////////////////////////////////////////////////////////////////////
// CSLIDEPrimitive Class
//

class CSLIDEWorld;

class CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEPrimitive();
  virtual ~CSLIDEPrimitive();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual VOID Preprocess();
  virtual VOID Update();

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();

//////////////////////////////////////////////////
// Rendering
//
public:

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile) = 0;

//////////////////////////////////////////////////
// Error Handling
//
public:
  VOID HandleError(CHAR *pcFormat, ...);
  VOID HandleWarning(CHAR *pcFormat, ...);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  static inline FILE *GetErrorFile();
  static inline VOID SetErrorFile(FILE *fpErrorFile);
  static inline UINT_32 GetErrorCount();
  static inline VOID SetErrorCount(UINT_32 uiErrors);
  static inline UINT_32 GetWarningCount();
  static inline VOID SetWarningCount(UINT_32 uiWarnings);

public:
  inline CHAR *GetFileName();
  inline UINT_32 GetFileLine();
  inline UINT_32 GetFileColumn();
  inline CHAR *GetID();

////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  // You will need to set this flag in the CheckDynamic() method if you
  // want the CSLIDEWorld to put it in its list of dynamic objects which
  // it updates on every time slice.
  inline BOOL GetDynamic();
  inline VOID SetDynamic(BOOL bDynamic);
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static FILE *m_fpErrorFile;
  static UINT_32 m_uiErrorCount;
  static UINT_32 m_uiWarningCount;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  // SLIDE .slf file position of the primitive for debugging the files
  CHAR *m_pcFile;
  UINT_32 m_uiLine;
  UINT_32 m_uiColumn;

  CHAR *m_pcID; // Stored in a string table so equals checks can be done on 
		// the pointer itself
  BOOL m_bDynamic; // Does this entity have any dynamic fields?
};

//////////////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline CHAR *CSLIDEPrimitive::GetFileName()
{
  return m_pcFile;
}

inline UINT_32 CSLIDEPrimitive::GetFileLine()
{
  return m_uiLine;
}

inline UINT_32 CSLIDEPrimitive::GetFileColumn()
{
  return m_uiColumn;
}

inline CHAR *CSLIDEPrimitive::GetID()
{
  return m_pcID;
}

inline BOOL CSLIDEPrimitive::GetDynamic()
{
  return m_bDynamic;
}

inline VOID CSLIDEPrimitive::SetDynamic(BOOL bDynamic)
{
  m_bDynamic = bDynamic;
}

//////////////////////////////////////////////////////////////////////
// CSLIDEPrimitiveIter Class
//

class CSLIDEPrimitiveIter
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEPrimitiveIter();
  virtual ~CSLIDEPrimitiveIter();
  virtual VOID Init(CSLIDEPrimitive *pCSLIDEPrimitive);
  virtual VOID Uninit();

  virtual BOOL Initialized();
  virtual BOOL Valid();

//////////////////////////////////////////////////
// Accessor Functions
//
public:

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CSLIDEPrimitive *m_pCSLIDEPrimitive;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline FILE *CSLIDEPrimitive::GetErrorFile()
{
  return m_fpErrorFile;
}

inline VOID CSLIDEPrimitive::SetErrorFile(FILE *fpErrorFile)
{
  ASSERT( fpErrorFile != NULL );

  m_fpErrorFile = fpErrorFile;
}

inline UINT_32 CSLIDEPrimitive::GetErrorCount()
{
  return m_uiErrorCount;
}
 
inline VOID CSLIDEPrimitive::SetErrorCount(UINT_32 uiErrors)
{
  m_uiErrorCount = uiErrors;
}

inline UINT_32 CSLIDEPrimitive::GetWarningCount()
{
  return m_uiWarningCount;
}

inline VOID CSLIDEPrimitive::SetWarningCount(UINT_32 uiWarnings)
{
  m_uiWarningCount = uiWarnings;
}

#endif __SLIDEPRIMITIVE_H__
