#ifndef __SLIDERENDER_H__
#define __SLIDERENDER_H__

#include "slideprimitive.h"

class CSLIDEWorld;
class CSLIDECameraPath;
class CSLIDENode;

class CSLIDETclVariables;

//////////////////////////////////////////////////////////////////////
// CSLIDERender Class
//

class CSLIDERender : public CSLIDEPrimitive
{
//////////////////////////////////////////////////
// Friends
//
public:

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDERender();
  virtual ~CSLIDERender();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcViewportID, CSLIDECameraPath *pCSLIDECameraPath,
		    CHAR *pcWorldRootID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual inline VOID SetTclTag(CHAR *pcTclTag);
  virtual inline VOID SetLOD(DynInt *pdyniLOD);

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

public:  

//////////////////////////////////////////////////
// Iteration Functions
//
public:

//////////////////////////////////////////////////
// Geometric Computations
//
public:
////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  virtual VOID PreprocessGraph();
  virtual VOID UpdateGraph();

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Input Functions
//
public:
  VOID HandleReshape(const Vector &vSize);
  BOOL HandleMouseButton(const Vector &vSize,
			 INT_32 iButton, INT_32 iState, 
			 const Vector &vClick);
  BOOL HandleMouseMotion(const Vector &vSize,
			 const Vector &vClick);

//////////////////////////////////////////////////
// Rendering
//
public:
////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  virtual VOID Render(const Vector &vOrigin, const Vector &vSize);
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:

//////////////////////////////////////////////////
// Class Variables
//
protected:
  static UINT_32 m_uiCount;

  static CListInt *m_plistCSLIDETclVariables;

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcTclTag; // Prefix attached the Tcl variables
  CSLIDETclVariables *m_pCSLIDETclVariables; // Tcl variables this updates

  CHAR *m_pcViewportID; // Name of the viewport which will point to this object

  CSLIDECameraPath *m_pCSLIDECameraPath; // Ignore this

  CHAR *m_pcWorldRootID;
  CSLIDENode *m_pWorldRoot; // CSLIDENode root of the scene graph

  DynInt m_dyniLOD; // Ignore this

  // UI Variables
  INT_32 m_iButtonCurr;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDERender::SetTclTag(CHAR *pcTclTag)
{
  m_pcTclTag = pcTclTag;
}

inline VOID CSLIDERender::SetLOD(DynInt *pdyniLOD)
{
  m_dyniLOD.Set(pdyniLOD);
}

//////////////////////////////////////////////////////////////////////
// CSLIDETclVariables Class
//

class CSLIDETclVariables
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDETclVariables();
  virtual ~CSLIDETclVariables();
  virtual VOID Init(CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();
  virtual BOOL Valid();

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline CHAR *GetID() const;

  inline FLOAT GetWidth() const;
  VOID SetWidth(FLOAT fWidth);
  inline FLOAT GetHeight() const;
  VOID SetHeight(FLOAT fHeight);
  inline FLOAT GetMouseX() const;
  VOID SetMouseX(FLOAT fMouseX);
  inline FLOAT GetMouseY() const;
  VOID SetMouseY(FLOAT fMouseY);

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcID;

  // Window Variables
  FLOAT m_fWidth;
  FLOAT m_fHeight;
  FLOAT m_fMouseX;
  FLOAT m_fMouseY;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline CHAR *CSLIDETclVariables::GetID() const
{
  return m_pcID;
}

inline FLOAT CSLIDETclVariables::GetWidth() const
{
  return m_fWidth;
}

inline FLOAT CSLIDETclVariables::GetHeight() const
{
  return m_fHeight;
}

inline FLOAT CSLIDETclVariables::GetMouseX() const
{
  return m_fMouseX;
}

inline FLOAT CSLIDETclVariables::GetMouseY() const
{
  return m_fMouseY;
}

#endif // __SLIDERENDER_H__
